@@ -871,9 +871,9 @@ gtk_source_completion_list_box_do_update (GtkSourceCompletionListBox *self,
 		}
 		else
 		{
+			gtk_widget_hide (GTK_WIDGET (iter));
 			_gtk_source_completion_list_box_row_display (GTK_SOURCE_COMPLETION_LIST_BOX_ROW (iter),
 			                                             NULL, NULL, NULL, self->show_icons, FALSE);
-			gtk_widget_hide (GTK_WIDGET (iter));
 		}
 
 		state.position++;
@@ -903,8 +903,20 @@ gtk_source_completion_list_box_update_cb (GtkWidget     *widget,
 
 	g_assert (GTK_SOURCE_IS_COMPLETION_LIST_BOX (self));
 
-	gtk_source_completion_list_box_do_update (self, TRUE);
 	self->queued_update = 0;
+
+	gtk_source_completion_list_box_do_update (self, TRUE);
+
+	/* There is a chance that the update sequence could cause us to need
+	 * to queue another update. But we don't actually need it. Just cancel
+	 * any additional request immediately.
+	 */
+	if (self->queued_update != 0)
+	{
+		gtk_widget_remove_tick_callback (GTK_WIDGET (self), self->queued_update);
+		self->queued_update = 0;
+	}
+
 	return G_SOURCE_REMOVE;
 }
 
@@ -1075,9 +1087,14 @@ gtk_source_completion_list_box_items_changed_cb (GtkSourceCompletionListBox *sel
 
 	offset = gtk_source_completion_list_box_get_offset (self);
 
-	/* Skip widget resize if results are after visible area */
-	if (position >= offset + self->n_rows)
+	/* Skip widget resize if results are out of view and wont force our
+	 * current results down.
+	 */
+	if ((position >= offset + self->n_rows) ||
+	    (removed == added && (position + added) < offset))
+	{
 		return;
+	}
 
 	gtk_source_completion_list_box_queue_update (self);
 }
